@@ -2,7 +2,7 @@
  * Linux-specific portion of Broadcom 802.11abg Networking Device Driver
  * cfg80211 interface
  *
- * Copyright (C) 2013, Broadcom Corporation. All Rights Reserved.
+ * Copyright (C) 2014, Broadcom Corporation. All Rights Reserved.
  * 
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -43,7 +43,11 @@
 #define EVENT_FLAGS(e) dtoh16((e)->flags)
 #define EVENT_STATUS(e) dtoh32((e)->status)
 
+#ifdef BCMDBG
 u32 wl_dbg_level = WL_DBG_ERR | WL_DBG_INFO;
+#else
+u32 wl_dbg_level = WL_DBG_ERR;
+#endif
 
 static s32 wl_cfg80211_change_iface(struct wiphy *wiphy, struct net_device *ndev,
            enum nl80211_iftype type, u32 *flags, struct vif_params *params);
@@ -1271,6 +1275,9 @@ wl_cfg80211_add_key(struct wiphy *wiphy, struct net_device *dev,
 	}
 
 	key_endian_to_device(&key);
+	if (wl->passive) {
+		schedule();
+	}
 	err = wl_dev_ioctl(dev, WLC_SET_KEY, &key, sizeof(key));
 	if (err) {
 		WL_ERR(("WLC_SET_KEY error (%d)\n", err));
@@ -1425,7 +1432,7 @@ wl_cfg80211_get_station(struct wiphy *wiphy, struct net_device *dev,
 	s32 err = 0;
 
 	if (memcmp(mac, wl->profile->bssid, ETHER_ADDR_LEN)) {
-		WL_ERR(("Wrong Mac address\n"));
+		WL_ERR(("Wrong Mac address, mac = %pM   profile =%pM\n", mac, wl->profile->bssid));
 		return -ENOENT;
 	}
 
@@ -1584,6 +1591,184 @@ wl_cfg80211_flush_pmksa(struct wiphy *wiphy, struct net_device *dev)
 
 #endif  
 
+#ifdef CONFIG_PM
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 39)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 9, 0)
+
+static int
+wl_wowl_ind_wake_reason(struct wl_cfg80211_priv *wl, struct cfg80211_wowlan_wakeup *wakeup)
+{
+	wl_wowl_wakeind_t wowl_ind;
+	s32 err;
+
+	err = wl_dev_bufvar_get(wl_to_ndev(wl), "wowl_wakeind",
+		(s8 *)&wowl_ind, sizeof(wowl_ind));
+	if (err != 0) {
+		WL_ERR(("Unable to get wake reason, err = %d\n", err));
+		return -1;
+	}
+
+	if (wowl_ind.ucode_wakeind == 0) {
+		WL_DBG(("System woke, but not by us\n"));
+		return 0;
+	}
+	WL_DBG(("wake reason is 0x%x\n", wowl_ind.ucode_wakeind));
+
+	if (wowl_ind.ucode_wakeind & WL_WOWL_MAGIC) {
+		WL_ERR(("WOWLAN Woke for: Magic Pkt\n"));
+		wakeup->magic_pkt = true;
+	}
+	if (wowl_ind.ucode_wakeind & WL_WOWL_DIS) {
+		WL_ERR(("WOWLAN Woke for: Disconnect\n"));
+		wakeup->disconnect = true;
+	}
+	if (wowl_ind.ucode_wakeind & WL_WOWL_BCN) {
+		WL_ERR(("WOWLAN Woke for: Beacon Loss\n"));
+		wakeup->disconnect = true;
+	}
+	if (wowl_ind.ucode_wakeind & WL_WOWL_GTK_FAILURE) {
+		WL_ERR(("WOWLAN Woke for: GTK failure\n"));
+		wakeup->gtk_rekey_failure = true;
+	}
+	if (wowl_ind.ucode_wakeind & WL_WOWL_EAPID) {
+		WL_ERR(("WOWLAN Woke for: EAP identify request\n"));
+		wakeup->eap_identity_req = true;
+	}
+	if (wowl_ind.ucode_wakeind & WL_WOWL_M1) {
+		WL_ERR(("WOWLAN Woke for: 4-way handshake request\n"));
+		wakeup->four_way_handshake = true;
+	}
+	return 1;
+}
+#endif 
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 1, 0)
+static int
+wl_cfg80211_rekey(struct wiphy *wiphy, struct net_device *ndev,
+        struct cfg80211_gtk_rekey_data *data)
+{
+	struct wl_cfg80211_priv *wl = wiphy_to_wl(wiphy);
+	wlc_rekey_info_t rekey;
+	s32 err;
+
+	if (!wl->offloads) {
+		return 0;
+	}
+
+	memset(&rekey, 0, sizeof(rekey));
+	memcpy(&rekey.kek, data->kek, WLC_KEK_LEN);
+	memcpy(&rekey.kck, data->kck, WLC_KCK_LEN);
+	memcpy(&rekey.replay_counter, data->replay_ctr, WLC_REPLAY_CTR_LEN);
+	WL_INF(("Send down replay counter %x%x%x%x%x%x%x%x\n",
+		rekey.replay_counter[0], rekey.replay_counter[1], rekey.replay_counter[2],
+		rekey.replay_counter[3], rekey.replay_counter[4], rekey.replay_counter[5],
+		rekey.replay_counter[6], rekey.replay_counter[7]));
+
+	err = wl_dev_bufvar_set(wl_to_ndev(wl), "wowl_replay", (s8 *)&rekey, sizeof(rekey));
+	if (err) {
+		WL_ERR(("Error calling wowl_set_key\n"));
+		return err;
+	}
+	return err;
+}
+#endif 
+
+static int wl_cfg80211_suspend(struct wiphy *wiphy, struct cfg80211_wowlan *wowlan)
+{
+	struct wl_cfg80211_priv *wl = wiphy_to_wl(wiphy);
+	uint wowl = 0;
+	s32 err;
+
+	if (!wowlan) {
+		WL_DBG(("No wowlan requested\n"));
+		return 0;
+	}
+	if (!test_bit(WL_STATUS_CONNECTED, &wl->status)) {
+		WL_INF(("No wowl when not associated.\n"));
+		return 0;
+	}
+
+	err = wl_dev_intvar_get(wl_to_ndev(wl), "wowl", &wowl);
+	if (err) {
+		WL_ERR(("Error fetching WOWL %d\n", err));
+	}
+	if (wowlan->disconnect) {
+		WL_INF(("Requesting wake on Disconnect\n"));
+		wowl |= WL_WOWL_DIS | WL_WOWL_BCN;
+	}
+	if (wowlan->magic_pkt) {
+		WL_INF(("Requesting wake on Magic Pkt\n"));
+		wowl |= WL_WOWL_MAGIC;
+	}
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 1, 0)
+	if (wowlan->gtk_rekey_failure) {
+		WL_INF(("Requesting wake GTK rekey failure Pkt\n"));
+		wowl |= WL_WOWL_GTK_FAILURE;
+	}
+	if (wowlan->four_way_handshake) {
+		WL_INF(("Requesting wake on 4way handshake request\n"));
+		wowl |= WL_WOWL_M1;
+	}
+#endif 
+
+	wowl |= WL_WOWL_KEYROT;
+
+	err = wl_dev_intvar_set(wl_to_ndev(wl), "wowl", wowl);
+	if (err) {
+		WL_ERR(("Error enabling WOWL %d\n", err));
+	}
+
+	return err;
+}
+#else 
+static int wl_cfg80211_suspend(struct wiphy *wiphy)
+{
+	return 0;
+}
+#endif 
+
+static int wl_cfg80211_resume(struct wiphy *wiphy)
+{
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 1, 0)
+	struct wl_cfg80211_priv *wl = wiphy_to_wl(wiphy);
+	wlc_rekey_info_t *rekey = (wlc_rekey_info_t *)wl->extra_buf;
+	s32 err;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 9, 0)
+	struct cfg80211_wowlan_wakeup wakeup;
+	int result;
+
+	memset(&wakeup, 0, sizeof(wakeup));
+	wakeup.pattern_idx = -1;
+
+	result = wl_wowl_ind_wake_reason(wl, &wakeup);
+	switch (result) {
+		case -1: 
+			break;
+		case 0:	
+			cfg80211_report_wowlan_wakeup(wl_to_wdev(wl), NULL, GFP_KERNEL);
+			break;
+		case 1: 
+			cfg80211_report_wowlan_wakeup(wl_to_wdev(wl), &wakeup, GFP_KERNEL);
+			break;
+	}
+#endif 
+
+	err = wl_dev_bufvar_get(wl_to_ndev(wl), "wowl_replay", (s8 *)rekey,
+		sizeof(wlc_rekey_info_t));
+	if (!err) {
+		WL_INF(("Send up replay counter %x%x%x%x%x%x%x%x\n",
+			rekey->replay_counter[0], rekey->replay_counter[1],
+			rekey->replay_counter[2], rekey->replay_counter[3],
+			rekey->replay_counter[4], rekey->replay_counter[5],
+			rekey->replay_counter[6], rekey->replay_counter[7]));
+		cfg80211_gtk_rekey_notify(wl_to_ndev(wl), (u8 *)&wl->bssid.octet,
+			rekey->replay_counter, GFP_KERNEL);
+	}
+#endif 
+	return 0;
+}
+#endif 
+
 static struct cfg80211_ops wl_cfg80211_ops = {
 	.change_virtual_intf = wl_cfg80211_change_iface,
 	.scan = wl_cfg80211_scan,
@@ -1600,6 +1785,13 @@ static struct cfg80211_ops wl_cfg80211_ops = {
 	.set_power_mgmt = wl_cfg80211_set_power_mgmt,
 	.connect = wl_cfg80211_connect,
 	.disconnect = wl_cfg80211_disconnect,
+#ifdef CONFIG_PM
+	.suspend = wl_cfg80211_suspend,
+	.resume = wl_cfg80211_resume,
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 1, 0)
+	.set_rekey_data = wl_cfg80211_rekey,
+#endif 
+#endif 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33)
 	.set_pmksa = wl_cfg80211_set_pmksa,
 	.del_pmksa = wl_cfg80211_del_pmksa,
@@ -1607,6 +1799,23 @@ static struct cfg80211_ops wl_cfg80211_ops = {
 #endif
 };
 
+#ifdef CONFIG_PM
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 39)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 11, 0)
+static const struct wiphy_wowlan_support wl_wowlan_support = {
+#else
+static struct wiphy_wowlan_support wl_wowlan_support = {
+#endif 
+	.flags = WIPHY_WOWLAN_MAGIC_PKT
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 1, 0)
+	| WIPHY_WOWLAN_SUPPORTS_GTK_REKEY | WIPHY_WOWLAN_GTK_REKEY_FAILURE |
+	WIPHY_WOWLAN_EAP_IDENTITY_REQ
+#endif
+	| WIPHY_WOWLAN_DISCONNECT,
+};
+#endif 
+#endif 
+
 static s32 wl_mode_to_nl80211_iftype(s32 mode)
 {
 	s32 err = 0;
@@ -1655,6 +1864,17 @@ static s32 wl_alloc_wdev(struct device *dev, struct wireless_dev **rwdev)
 
 	wdev->wiphy->flags &= ~WIPHY_FLAG_PS_ON_BY_DEFAULT;
 #endif
+
+#ifdef CONFIG_PM
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 39)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 11, 0)
+	wdev->wiphy->wowlan = &wl_wowlan_support;
+#else
+	wdev->wiphy->wowlan = wl_wowlan_support;
+#endif 
+#endif 
+#endif 
+
 	err = wiphy_register(wdev->wiphy);
 	if (err < 0) {
 		WL_ERR(("Couldn not register wiphy device (%d)\n", err));
@@ -1763,7 +1983,17 @@ static s32 wl_inform_single_bss(struct wl_cfg80211_priv *wl, struct wl_bss_info
 #else
 	freq = ieee80211_channel_to_frequency(notif_bss_info->channel);
 #endif
+	if (freq == 0) {
+		WL_ERR(("Invalid channel, fail to chcnage channel to freq\n"));
+		kfree(notif_bss_info);
+		return -EINVAL;
+	}
 	channel = ieee80211_get_channel(wiphy, freq);
+	if (unlikely(!channel)) {
+		WL_ERR(("ieee80211_get_channel error\n"));
+		kfree(notif_bss_info);
+		return -EINVAL;
+	}
 
 	WL_DBG(("SSID : \"%s\", rssi %d, channel %d, capability : 0x04%x, bssid %pM\n",
 		bi->SSID, notif_bss_info->rssi, notif_bss_info->channel,
@@ -1902,7 +2132,7 @@ wl_dev_bufvar_get(struct net_device *dev, s8 *name, s8 *buf, s32 buf_len)
 	BUG_ON(!len);
 	err = wl_dev_ioctl(dev, WLC_GET_VAR, (void *)wl->ioctl_buf, WL_IOCTL_LEN_MAX);
 	if (err) {
-		WL_ERR(("error (%d)\n", err));
+		WL_INF(("error (%d)\n", err));
 		return err;
 	}
 	memcpy(buf, wl->ioctl_buf, buf_len);
@@ -2046,6 +2276,7 @@ static s32 wl_update_bss_info(struct wl_cfg80211_priv *wl)
 		ie = bss->information_elements;
 		ie_len = bss->len_information_elements;
 #endif
+		wl->conf->channel = *bss->channel;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 9, 0)
 		cfg80211_put_bss(wiphy, bss);
 #else
@@ -2079,11 +2310,12 @@ wl_bss_roaming_done(struct wl_cfg80211_priv *wl, struct net_device *ndev,
 	s32 err = 0;
 
 	wl_get_assoc_ies(wl);
+	memcpy(wl->profile->bssid, &e->addr, ETHER_ADDR_LEN);
 	memcpy(&wl->bssid, &e->addr, ETHER_ADDR_LEN);
 	wl_update_bss_info(wl);
 	cfg80211_roamed(ndev,
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 39)
-			NULL,	 
+			&wl->conf->channel,	 
 #endif
 			(u8 *)&wl->bssid,
 			conn_info->req_ie, conn_info->req_ie_len,
@@ -2102,27 +2334,28 @@ wl_bss_connect_done(struct wl_cfg80211_priv *wl, struct net_device *ndev,
 	struct wl_cfg80211_connect_info *conn_info = wl_to_conn(wl);
 	s32 err = 0;
 
-	wl_get_assoc_ies(wl);
-	memcpy(&wl->bssid, &e->addr, ETHER_ADDR_LEN);
-	wl_update_bss_info(wl);
+	if (wl->scan_request) {
+		WL_DBG(("%s: Aborting scan\n", __FUNCTION__));
+		cfg80211_scan_done(wl->scan_request, true);     
+		wl->scan_request = NULL;
+	}
 
-	WL_DBG(("Reporting BSS network join result \"%s\"\n", wl->profile->ssid.SSID));
 	if (test_and_clear_bit(WL_STATUS_CONNECTING, &wl->status)) {
+		if (completed) {
+			wl_get_assoc_ies(wl);
+			memcpy(&wl->bssid, &e->addr, ETHER_ADDR_LEN);
+			memcpy(wl->profile->bssid, &e->addr, ETHER_ADDR_LEN);
+			wl_update_bss_info(wl);
+			set_bit(WL_STATUS_CONNECTED, &wl->status);
+		}
+
+		WL_DBG(("Reporting BSS network join result \"%s\"\n",
+			wl->profile->ssid.SSID));
 		cfg80211_connect_result(ndev, (u8 *)&wl->bssid,	conn_info->req_ie,
 		    conn_info->req_ie_len, conn_info->resp_ie, conn_info->resp_ie_len,
 		    completed ? WLAN_STATUS_SUCCESS : WLAN_STATUS_AUTH_TIMEOUT,	GFP_KERNEL);
-		WL_DBG(("connection %s\n", completed ? "succeeded" : "failed"));
+		WL_DBG(("Connection %s\n", completed ? "Succeeded" : "FAILed"));
 	}
-	else {
-		cfg80211_roamed(ndev,
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 39)
-			NULL, 
-#endif
-		     (u8 *)&wl->bssid,	conn_info->req_ie, conn_info->req_ie_len,
-		     conn_info->resp_ie, conn_info->resp_ie_len,	GFP_KERNEL);
-		WL_DBG(("roaming result\n"));
-	}
-	set_bit(WL_STATUS_CONNECTED, &wl->status);
 
 	return err;
 }
@@ -2356,7 +2589,7 @@ static void wl_deinit_cfg80211_priv(struct wl_cfg80211_priv *wl)
 	wl_deinit_priv_mem(wl);
 }
 
-s32 wl_cfg80211_attach(struct net_device *ndev, struct device *dev)
+s32 wl_cfg80211_attach(struct net_device *ndev, struct device *dev, int passive)
 {
 	struct wireless_dev *wdev;
 	struct wl_cfg80211_priv *wl;
@@ -2382,6 +2615,7 @@ s32 wl_cfg80211_attach(struct net_device *ndev, struct device *dev)
 		WL_ERR(("Failed to init iwm_priv (%d)\n", err));
 		goto cfg80211_attach_out;
 	}
+	wl->passive = !!passive;
 
 	if (!err) {
 		WL_INF(("Registered CFG80211 phy\n"));
@@ -2420,8 +2654,8 @@ static s32 wl_event_handler(void *data)
 			WL_ERR(("eqeue empty..\n"));
 			BUG();
 		}
-		WL_DBG(("event type (%d)\n", e->etype));
 		if (wl->el.handler[e->etype]) {
+			WL_DBG(("event type (%d)\n", e->etype));
 			wl->el.handler[e->etype] (wl, wl_to_ndev(wl), &e->emsg, e->edata);
 		} else {
 			WL_DBG(("Unknown Event (%d): ignoring\n", e->etype));
@@ -2550,6 +2784,32 @@ static s32 wl_set_mode(struct net_device *ndev, s32 iftype)
 	return 0;
 }
 
+static void wl_update_wowl(struct net_device *ndev)
+{
+#ifdef CONFIG_PM
+	struct wl_cfg80211_priv *wl = ndev_to_wl(ndev);
+	struct wireless_dev *wdev = ndev->ieee80211_ptr;
+	s32 offloads = 0;
+	s32 err = 0;
+	err = wl_dev_bufvar_get(wl_to_ndev(wl), "offloads",
+		(s8 *)&offloads, sizeof(offloads));
+	if (err == 0 && offloads == 1) {
+		WL_INF(("Supports offloads\n"));
+		wl->offloads = true;
+	} else {
+		WL_INF(("No offloads supported\n"));
+		wl->offloads = false;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 39)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 11, 0)
+		wdev->wiphy->wowlan = NULL;
+#else
+		memset(&wdev->wiphy->wowlan, 0, sizeof(struct wiphy_wowlan_support));
+#endif 
+#endif 
+	}
+#endif 
+}
+
 static s32 wl_update_wiphybands(struct wl_cfg80211_priv *wl)
 {
 	struct wiphy *wiphy;
@@ -2566,7 +2826,7 @@ static s32 wl_update_wiphybands(struct wl_cfg80211_priv *wl)
 	phy = ((char *)&phy_list)[0];
 	WL_DBG(("%c phy\n", phy));
 
-	if (phy == 'n' || phy == 'a') {
+	if (phy == 'n' || phy == 'a' || phy == 'v') {
 		wiphy = wl_to_wiphy(wl);
 		wiphy->bands[IEEE80211_BAND_5GHZ] = &__wl_band_5ghz_n;
 	}
@@ -2581,10 +2841,13 @@ s32 wl_cfg80211_up(struct net_device *ndev)
 	struct wireless_dev *wdev = ndev->ieee80211_ptr;
 
 	wl_set_mode(ndev, wdev->iftype);
-
 	err = wl_update_wiphybands(wl);
+	if (err) {
+		return err;
+	} 
 
-	return err;
+	wl_update_wowl(ndev);
+	return 0;
 }
 
 s32 wl_cfg80211_down(struct net_device *ndev)
